'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');

const flattenVNodes = (children1)=>{
    const result = [];
    const traverse = (children)=>{
        if (Array.isArray(children)) {
            children.forEach((child)=>{
                if (vue.isVNode(child)) {
                    var ref;
                    result.push(child);
                    if ((ref = child.component) === null || ref === void 0 ? void 0 : ref.subTree) {
                        traverse(child.component.subTree.children);
                    }
                    if (child.children) {
                        traverse(child.children);
                    }
                }
            });
        }
    };
    traverse(children1);
    return result;
};
const sortChildren = (parent, publicChildren, internalChildren)=>{
    const VNodes = flattenVNodes(parent.subTree.children);
    internalChildren.sort(// eslint-disable-next-line comma-dangle
    (a, b)=>VNodes.indexOf(a.vnode) - VNodes.indexOf(b.vnode));
    const orderedPublicChildren = internalChildren.map((item)=>item.proxy);
    publicChildren.sort((a, b)=>{
        const indexA = orderedPublicChildren.indexOf(a);
        const indexB = orderedPublicChildren.indexOf(b);
        return indexA - indexB;
    });
};
const createProvider = (key)=>{
    const publicChildren = vue.reactive([]);
    const internalChildren = vue.reactive([]);
    const parent = vue.getCurrentInstance();
    const provider = (value)=>{
        const link = (child)=>{
            if (child.proxy) {
                internalChildren.push(child);
                publicChildren.push(child.proxy);
                sortChildren(parent, publicChildren, internalChildren);
            }
        };
        const unlink = (child)=>{
            const idx = internalChildren.indexOf(child);
            publicChildren.splice(idx, 1);
            internalChildren.splice(idx, 1);
        };
        vue.provide(key, Object.assign({
            link,
            unlink,
            children: publicChildren,
            internalChildren
        }, value));
    };
    return {
        children: publicChildren,
        provider
    };
};

exports.createProvider = createProvider;
exports.flattenVNodes = flattenVNodes;
exports.sortChildren = sortChildren;
